package ruirossi.editora.servlets;

import ruirossi.editora.beans.*;
import ruirossi.editora.erros.*;
import ruirossi.editora.negocio.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class Controlador extends HttpServlet {
  protected void doGet(HttpServletRequest req,
    HttpServletResponse res) throws ServletException, IOException {
    processar(req, res);
  }
  
  protected void doPost(HttpServletRequest req,
    HttpServletResponse res) throws ServletException, IOException {
    processar(req, res);
  }
  
  private void processar(HttpServletRequest req, 
    HttpServletResponse res) throws ServletException, IOException {
    String url = "/index.jsp";
    String operacao = req.getParameter("operacao");
    if (operacao == null) operacao = "inicio";
    
    try {
      ServletContext sc = getServletContext();
      if (operacao.equals("logon")) {
        url = logon(req);
        sc.getRequestDispatcher(url).forward(req,res);
        return;
      }
      else if (operacao.equals("logoff")) {
        url = logoff(req);
        sc.getRequestDispatcher(url).forward(req,res);
        return;
      }
      else verificarPermissao(req);
      
      if (operacao.equals("funcionario_incluir"))
        url = incluirFuncionario(req);
      else if (operacao.equals("funcionario_lista"))
        url = listarFuncionarios(req);
      else if (operacao.equals("funcionario_excluir"))
        url = excluirFuncionario(req);
      else if (operacao.equals("assunto_incluir"))
        url = incluirAssunto(req);
      else if (operacao.equals("assunto_lista"))
        url = listarAssuntos(req);
      else if (operacao.equals("assunto_excluir"))
        url = excluirAssunto(req);
      else if (operacao.equals("autor_incluir"))
        url = incluirAutor(req);
      else if (operacao.equals("autor_lista"))
        url = listarAutores(req);
      else if (operacao.equals("autor_excluir"))
        url = excluirAutor(req);
      else if (operacao.equals("livro_novo"))
        url = novoLivro(req);
      else if (operacao.equals("livro_incluir"))
        url = incluirLivro(req);
      else if (operacao.equals("livro_lista"))
        url = listarLivros(req);
      else if (operacao.equals("livro_excluir"))
        url = excluirLivro(req);
      else if (operacao.equals("cliente_lista"))
        url = listarClientes(req);
      else if (operacao.equals("cliente_excluir"))
        url = excluirCliente(req);
      else throw new EditoraException("Operao invlida");
    }
    catch (Exception ex) {
      req.setAttribute("erro",ex);
      url = "/erro_tratado.jsp";
    }
    
    getServletContext().getRequestDispatcher(url).forward(req,res);
  }
  
  private String logon(HttpServletRequest req)
    throws EditoraException {
    FuncionarioBean b = new FuncionarioBean();
    b.setLogin(req.getParameter("login"));
    b.setSenha(req.getParameter("senha"));
    b = FuncionarioBO.validar(b);
    if (b == null) throw new EditoraException("Dados invlidos!");
    req.getSession().setAttribute("funcionario",b);
    return "/index.jsp";
  }
  
  private String logoff(HttpServletRequest req)
    throws EditoraException {
    req.getSession().invalidate();
    return "/index.jsp";
  }
  
  private FuncionarioBean verificarPermissao(HttpServletRequest req)
    throws EditoraException {
    HttpSession sessao = req.getSession();
    FuncionarioBean b = null;
    b = (FuncionarioBean)sessao.getAttribute("funcionario");
    String msg = "Efetue o logon antes de realizar esta operao";
    if (b == null) throw new EditoraException(msg);
    return b;
  }
  
  private String incluirFuncionario(HttpServletRequest req)
    throws EditoraException {
    FuncionarioBean b = new FuncionarioBean(req.getParameter("login"),
      req.getParameter("senha"),req.getParameter("nome"));
    FuncionarioBO.incluir(b);
    req.setAttribute("url","funcionario_incluir.jsp");
    return "/sucesso.jsp";
  }
  
  private String listarFuncionarios(HttpServletRequest req)
    throws EditoraException {
    req.setAttribute("funcionario_lista",FuncionarioBO.listar());
    return "/funcionario_lista.jsp";
  }
  
  private String excluirFuncionario(HttpServletRequest req)
    throws EditoraException {
    FuncionarioBean b = new FuncionarioBean();
    b.setLogin(req.getParameter("login"));
    FuncionarioBO.excluir(b);
    Collection co = FuncionarioBO.listar();
    req.setAttribute("funcionario_lista",co);
    return "/funcionario_lista.jsp";
  }
  
  private String incluirAssunto(HttpServletRequest req)
    throws EditoraException {
    AssuntoBean b = new AssuntoBean();
    b.setDescricao(req.getParameter("descricao"));
    AssuntoBO.incluir(b);
    req.setAttribute("url","assunto_incluir.jsp");
    return "/sucesso.jsp";
  }
  
  private String listarAssuntos(HttpServletRequest req)
    throws EditoraException {
    req.setAttribute("assunto_lista",AssuntoBO.listar());
    return "/assunto_lista.jsp";
  }

  private String excluirAssunto(HttpServletRequest req)
    throws EditoraException {
    AssuntoBean b = new AssuntoBean();
    b.setCodigo(req.getParameter("codigo"));
    AssuntoBO.excluir(b);
    req.setAttribute("assunto_lista",AssuntoBO.listar());
    return "/assunto_lista.jsp";
  }
      
  private String incluirAutor(HttpServletRequest req)
    throws EditoraException {
    AutorBean b = new AutorBean();
    b.setNome(req.getParameter("nome"));
    AutorBO.incluir(b);
    req.setAttribute("url","autor_incluir.jsp");
    return "/sucesso.jsp";
  }
      
  private String listarAutores(HttpServletRequest req)
    throws EditoraException {
    Collection co = AutorBO.listar();
    req.setAttribute("autor_lista",co);
    return "/autor_lista.jsp";
  }
      
  private String excluirAutor(HttpServletRequest req)
    throws EditoraException {
    AutorBean b = new AutorBean();
    b.setCodigo(req.getParameter("codigo"));
    AutorBO.excluir(b);
    Collection co = AutorBO.listar();
    req.setAttribute("autor_lista",co);
    return "/autor_lista.jsp";
  }
      
  private String novoLivro(HttpServletRequest req)
    throws EditoraException {
    req.setAttribute("assunto_lista",AssuntoBO.listar());
    req.setAttribute("autor_lista",AutorBO.listar());
    return "/livro_incluir.jsp";
  }
      
  private String incluirLivro(HttpServletRequest req)
    throws EditoraException {
    LivroBean b = new LivroBean();
    b.setTitulo(req.getParameter("titulo"));
    b.setPreco(req.getParameter("preco"));
    b.setAno(req.getParameter("ano"));
    b.setPaginas(req.getParameter("paginas"));
    b.setAssunto(AssuntoBO.selecionar(req.getParameter("assunto")));
    
    Vector co_autores = new Vector();
    
    String[] autores = req.getParameterValues("autores");
    
    if (autores == null || autores.length == 0)
      throw new EditoraException("Selecione ao menos um autor");
    
    for (int i = 0; i < autores.length; i++) 
      co_autores.add(AutorBO.selecionar(autores[i]));
    
    b.setAutores(co_autores);
    
    LivroBO.incluir(b);
    
    req.setAttribute("url","main?operacao=livro_novo");
    return "/sucesso.jsp";
  }
      
  private String listarLivros(HttpServletRequest req)
    throws EditoraException {
    Collection co = LivroBO.listar();
    req.setAttribute("livro_lista",co);
    return "/livro_lista.jsp";
  }
      
  private String excluirLivro(HttpServletRequest req)
    throws EditoraException {
    LivroBean b = new LivroBean();
    b.setCodigo(req.getParameter("codigo"));
    LivroBO.excluir(b);
    Collection co = LivroBO.listar();
    req.setAttribute("livro_lista",co);
    return "/livro_lista.jsp";
  }
      
  private String listarClientes(HttpServletRequest req)
    throws EditoraException {
    Collection co = ClienteBO.listar();
    req.setAttribute("cliente_lista",co);
    return "/cliente_lista.jsp";
  }
      
  private String excluirCliente(HttpServletRequest req)
    throws EditoraException {
    ClienteBean b = new ClienteBean();
    b.setEmail(req.getParameter("email"));
    ClienteBO.excluir(b);
    Collection co = ClienteBO.listar();
    req.setAttribute("cliente_lista",co);
    return "/cliente_lista.jsp";
  }
}