package ruirossi.editora.beans;

import ruirossi.editora.erros.*;
import java.util.*;

public class LivroBean {
  private int codigo;
  private String titulo;
  private double preco;
  private int ano;
  private int paginas;
  private AssuntoBean assunto;
  private Collection autores;
  
  public LivroBean() { }
  
  public LivroBean(String codigo, String titulo, String preco, 
    String ano, String paginas, AssuntoBean assunto, 
    Collection autores) throws EditoraException {
    setCodigo(codigo);
    setTitulo(titulo);
    setPreco(preco);
    setAno(ano);
    setPaginas(paginas);
    setAssunto(assunto);
    setAutores(autores);
  }
  
  public int getCodigo() {return codigo;}
  public String getTitulo() {return titulo;}
  public double getPreco() {return preco;}
  public int getAno() {return ano;}
  public int getPaginas() {return paginas;}
  public AssuntoBean getAssunto() {return assunto;}
  public Collection getAutores() {return autores;}
  
  public void setCodigo(int codigo) throws EditoraException {
    if (codigo <= 0) 
      throw new EditoraException("O cdigo deve ser maior que zero");
    this.codigo = codigo;
  }
  
  public void setCodigo(String codigo) throws EditoraException {
    if (codigo == null)
      throw new EditoraException("O cdigo no pode ser nulo");
    try {setCodigo(Integer.parseInt(codigo));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Cdigo invlido: " + codigo);
    }
  }
  
  public void setTitulo(String titulo) throws EditoraException {
    if (titulo == null)
      throw new EditoraException("O ttulo no pode ser nulo");
    titulo = titulo.trim();
    if (titulo.length() == 0)
      throw new EditoraException(" preciso informar o ttulo");
    if (titulo.length() > 100)
      throw new EditoraException("Ttulo com mais de 100 caracteres");
    this.titulo = titulo;
  }
  
  public void setPreco(double preco) throws EditoraException {
    if (preco <= 0)
      throw new EditoraException("O preo deve ser maior que zero");
    this.preco = preco;
  }
  
  public void setPreco(String preco) throws EditoraException {
    if (preco == null)
      throw new EditoraException("O preo no pode ser nulo");
    try {setPreco(Double.parseDouble(preco));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Preo invlido: " + preco);
    }
  }
  
  public void setAno(int ano) throws EditoraException {
    if (ano < 0)
      throw new EditoraException("O ano deve ser um nmero positivo");
    this.ano = ano;
  }
  
  public void setAno(String ano) throws EditoraException {
    if (ano == null)
      throw new EditoraException("O ano no pode ser nulo");
    try {setAno(Integer.parseInt(ano));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Ano invlido: " + ano);
    }
  }
  
  public void setPaginas(int paginas) throws EditoraException {
    if (paginas <= 0)
      throw new EditoraException("Nmero invlido de pginas");
    this.paginas = paginas;
  }
  
  public void setPaginas(String paginas) throws EditoraException {
    if (paginas == null)
      throw new EditoraException("Nmero de pginas nulo");
    try {setPaginas(Integer.parseInt(paginas));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Nmero invlido de pginas");
    }
  }
  
  public void setAssunto(AssuntoBean assunto) throws EditoraException{
    if (assunto == null)
      throw new EditoraException("O assunto no pode ser nulo");
    this.assunto = assunto;
  }
  
  public void setAutores(Collection autores) throws EditoraException {
    if (autores == null)
      throw new EditoraException("A lista de autores est vazia");
    if (autores.size() == 0)
      throw new EditoraException("Nenhum autor especificado");
    this.autores = autores;
    }
}