package ruirossi.editora.beans;

import ruirossi.editora.erros.*;
import java.text.*;
import java.util.*;

public class PedidoBean {
  private int numero;
  private Date data;
  private Date horario;
  private ClienteBean cliente;
  private Collection itens;
  
  public PedidoBean() { }
  
  public PedidoBean(String numero, String data, String horario,
    ClienteBean cliente, Collection itens) 
    throws EditoraException {
    setNumero(numero);
    setData(data);
    setHorario(horario);
    setCliente(cliente);
    setItens(itens);
  }
  
  public int getNumero() {return numero;}
  public Date getData() {return data;}
  public Date getHorario() {return horario;}
  public ClienteBean getCliente() {return cliente;}
  public Collection getItens() {return itens;}
  
  public void setNumero(int numero) throws EditoraException {
    if (numero <= 0) 
      throw new EditoraException("O nmero deve ser maior que zero");
    this.numero = numero;
  }
  
  public void setNumero(String numero) throws EditoraException {
    if (numero == null)
      throw new EditoraException("O nmero no pode ser nulo");
    try {setNumero(Integer.parseInt(numero));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Nmero invlido: " + numero);
    }
  }
  
  public void setData(Date data) throws EditoraException {
    if (data == null)
      throw new EditoraException("A data no pode ser nula");
    if (data.after(new Date()))
      throw new EditoraException("Data posterior  data atual");
    this.data = data;
  }
  
  public void setData(String data) throws EditoraException {
    if (data == null)
      throw new EditoraException("A data no pode ser nula");
    SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy");
    df.setLenient(false);
    try {setData(df.format(data));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Data invlida: " + data);
    }
  }
  
  public void setHorario(Date horario) throws EditoraException {
    if (horario == null)
      throw new EditoraException("O horrio no pode ser nulo");
    this.horario = horario;
  }
  
  public void setHorario(String horario) throws EditoraException {
    if (horario == null)
      throw new EditoraException("O horrio no pode ser nulo");
    SimpleDateFormat df = new SimpleDateFormat("HH:mm");
    df.setLenient(false);
    try {setHorario(df.format(horario));}
    catch (NumberFormatException nf) {
      throw new EditoraException("Horrio invlido: " + horario);
    }
  }
  
  public void setCliente(ClienteBean cliente) throws EditoraException{
    if (cliente == null)
      throw new EditoraException("O cliente no pode ser nulo");
    this.cliente = cliente;
  }
  
  public void setItens(Collection itens) throws EditoraException {
    if (itens == null)
      throw new EditoraException("Lista de itens nula");
    if (itens.size() == 0) 
      throw new EditoraException("Nenhum item especificado");
    this.itens = itens;
    }
}