package ruirossi.editora.persistencia;

import ruirossi.editora.erros.*;
import java.sql.*;
import java.util.Vector;

public class ConexaoMySQL {
  public static final int ACAO_EXECUTAR = 0;
  public static final int ACAO_CONFIRMAR = 1;
  public static final int ACAO_FECHAR = 2;
  
  private Connection con;
  private PreparedStatement pst;
  
  public ConexaoMySQL() throws EditoraException {
    String driver, url, usuario, senha;
    
    driver = "com.mysql.jdbc.Driver";
    url = "jdbc:mysql://localhost:3306/editora";
    usuario = "root";
    senha = "root";
    
    try {
      Class.forName(driver);
      con = DriverManager.getConnection(url,usuario,senha);
      con.setAutoCommit(false);
    }
    catch (ClassNotFoundException cnf) {
      String msg = "Driver de conexo com o banco no encontrado" +
        "\nMensagem original: " + cnf.getMessage();
      throw new EditoraException(msg);
    }
    catch (SQLException sql) {
      String msg = "No foi possvel abrir a conexo com o banco" +
        "\nMensagem original: " + sql.getMessage();
      throw new EditoraException(msg);
    }
  }
  
  public void prepararPst(String sql) throws EditoraException {
    try {pst = con.prepareStatement(sql);}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void executarPst(int acao) throws EditoraException{
    try {
      pst.execute();
      if (acao > 0) con.commit();
      if (acao > 1) con.close();
    }
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void setParam(int i, int p) throws EditoraException{
    try {pst.setInt(i, p);}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void setParam(int i, double p) throws EditoraException{
    try {pst.setDouble(i, p);}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void setParam(int i, String p) throws EditoraException{
    try {pst.setString(i, p);}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void setParam(int i, Date p) throws EditoraException{
    try {pst.setDate(i, p);}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void setParam(int i, Time p) throws EditoraException{
    try {pst.setTime(i, p);}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public void confirmar() throws EditoraException {
    try {con.commit();}
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
  
  public Vector selecionar () throws EditoraException {
    try {
      ResultSet rs = pst.executeQuery();
      if (rs == null) throw new EditoraException("Resultado vazio");
      ResultSetMetaData md = rs.getMetaData();
      Vector dados = new Vector();
      while (rs.next()) {
        Vector linha = new Vector();
        for (int i = 1; i <= md.getColumnCount(); i++)
          linha.add(rs.getString(i));
      dados.add(linha);
      }
      return dados;
    }
    catch (SQLException se) {
      throw new EditoraException("Erro de persistncia" +
        "\nMensagem original:  " + se.getMessage());
    }
  }
}