package ruirossi.editora.persistencia;

import ruirossi.editora.beans.*;
import ruirossi.editora.erros.*;
import ruirossi.editora.negocio.*;
import java.sql.*;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

public class LivroMySQL implements LivroDAO{
  public void incluir (LivroBean b) throws EditoraException {
    ConexaoMySQL con = new ConexaoMySQL();
    String sql = "INSERT INTO LIVRO (TITULO,PRECO,ANO,PAGINAS," +
      "ID_ASSUNTO) VALUES (?,?,?,?,?)";
    con.prepararPst(sql);
    con.setParam(1,b.getTitulo());
    con.setParam(2,b.getPreco());
    con.setParam(3,b.getAno());
    con.setParam(4,b.getPaginas());
    con.setParam(5,b.getAssunto().getCodigo());
    con.executarPst(con.ACAO_EXECUTAR);
    
    sql = "SELECT  MAX(CODIGO) AS MAXIMO FROM LIVRO";
    con.prepararPst(sql);
    Vector dados = con.selecionar();
    int codigo_livro = 0;
    if (dados.size() > 0) {
      Vector linha = (Vector)dados.get(0);
      codigo_livro = Integer.parseInt(linha.get(0).toString());
    }
    else throw new EditoraException("Gravao do livro falhou");
    
    Iterator i = b.getAutores().iterator();
    while (i.hasNext()) {
      AutorBean ab = (AutorBean)i.next();
    
      sql = "INSERT INTO LIVRO_AUTOR VALUES (?,?)";
      con.prepararPst(sql);
      con.setParam(1,codigo_livro);
      con.setParam(2,ab.getCodigo());
      con.executarPst(con.ACAO_EXECUTAR);
    }
    con.confirmar();
  }
  
  public void excluir (LivroBean b) throws EditoraException {
    ConexaoMySQL con = new ConexaoMySQL();
    String sql = "DELETE FROM LIVRO_AUTOR WHERE ID_LIVRO = ?";
    con.prepararPst(sql);
    con.setParam(1,b.getCodigo());
    con.executarPst(con.ACAO_EXECUTAR);
    sql = "DELETE FROM LIVRO WHERE CODIGO = ?";
    con.prepararPst(sql);
    con.setParam(1,b.getCodigo());
    con.executarPst(con.ACAO_FECHAR);
  }
  
  public Collection listar () throws EditoraException {
    String sql = "SELECT * FROM LIVRO ORDER BY TITULO";
    return getDados(sql);
  }
  
  public LivroBean selecionar (int codigo) throws EditoraException {
    ConexaoMySQL con = new ConexaoMySQL();
    String sql = "SELECT * FROM LIVRO WHERE CODIGO = ?";
    con.prepararPst(sql);
    con.setParam(1,codigo);
    Vector dados = con.selecionar();
    
    LivroBean b = new LivroBean();
    if (dados.size() > 0) {
      Vector li = (Vector)dados.get(0);
      b.setCodigo(li.get(0).toString());
      b.setTitulo(li.get(1).toString());
      b.setPreco(li.get(2).toString());
      b.setAno(li.get(3).toString());
      b.setPaginas(li.get(4).toString());
      AssuntoBean ab = new AssuntoBean();
      ab.setCodigo(li.get(5).toString());
      b.setAssunto(AssuntoBO.selecionar(ab.getCodigo()));
    }
    else throw new EditoraException("Livro no existe: " + codigo);
    
    sql = "SELECT A.* FROM AUTOR A, LIVRO_AUTOR " +
      "WHERE (A.CODIGO = LIVRO_AUTOR.ID_AUTOR) " +
      "AND (LIVRO_AUTOR.ID_LIVRO = ?)";
    con.prepararPst(sql);
    con.setParam(1,b.getCodigo());
    Vector autores = con.selecionar();
    Collection co_autores = new Vector();
    
    for (int k = 0; k < autores.size(); k++) {
      Vector autor = (Vector)autores.get(k);
      co_autores.add(new AutorBean(autor.get(0).toString(),
        autor.get(1).toString()));
    }
    
    b.setAutores(co_autores);
    return b;
  }
  
  public Collection selecionar(String titulo) throws EditoraException{
    String sql = "SELECT * FROM LIVRO WHERE UPPER(TITULO) " +
      "LIKE '%" + titulo.toUpperCase() + "%'";
    return getDados(sql);
  }
  
  public Collection selecionar (AssuntoBean assunto) 
    throws EditoraException {
    String sql = "SELECT * FROM LIVRO WHERE ID_ASSUNTO = " +
      assunto.getCodigo();
    return getDados(sql);
  }
  
  public Collection selecionar (AutorBean autor) 
    throws EditoraException {
    String sql = "SELECT * FROM LIVRO WHERE CODIGO IN " +
      "(SELECT ID_LIVRO FROM LIVRO_AUTOR WHERE ID_AUTOR IN " +
      "(SELECT CODIGO FROM AUTOR WHERE UPPER(NOME) LIKE '%" +
      autor.getNome().toUpperCase() + "%'))";
    return getDados(sql);
  }
  
  public Collection getDados(String sql) throws EditoraException {
    ConexaoMySQL con = new ConexaoMySQL();
    con.prepararPst(sql);
    Vector dados = con.selecionar();
    
    Collection resposta = new Vector();
    for (int i = 0; i < dados.size(); i++) {
      Vector li = (Vector)dados.get(i);
      LivroBean b = new LivroBean();
      b.setCodigo(li.get(0).toString());
      b = selecionar(b.getCodigo());
      resposta.add(b);
    }
    return resposta;
  }
}