package ruirossi.editora.servlets;

import ruirossi.editora.beans.*;
import ruirossi.editora.erros.*;
import ruirossi.editora.negocio.*;
import java.io.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;

public class Controlador extends HttpServlet {
  protected void doGet(HttpServletRequest req,
    HttpServletResponse res) throws ServletException, IOException {
    processar(req, res);
  }
  
  protected void doPost(HttpServletRequest req,
    HttpServletResponse res) throws ServletException, IOException {
    processar(req, res);
  }
  
  private void processar(HttpServletRequest req, 
    HttpServletResponse res) throws ServletException, IOException {
    String url = "/index.jsp";
    String operacao = req.getParameter("operacao");
    if (operacao == null) operacao = "inicio";
    
    try {
      if (operacao.equals("inicio")) url = carregarInicio(req);
      else if (operacao.equals("menu_assunto"))
        url = carregarAssunto(req);
      else if (operacao.equals("novo_cadastro"))
        url = "/cliente_incluir.jsp";
      else if (operacao.equals("cliente_incluir"))
        incluirCliente(req);
      else if (operacao.equals("cliente_excluir"))
        url = excluirCliente(req);
      else if (operacao.equals("logon"))
        url = logon(req);
      else if (operacao.equals("logoff"))
        url = logoff(req);
      else if (operacao.equals("carrinho_exibir"))
        url = exibirCarrinho(req);
      else if (operacao.equals("carrinho_incluir"))
        url = incluirNoCarrinho(req);
      else if (operacao.equals("carrinho_excluir"))
        url = excluirDoCarrinho(req);
      else if (operacao.equals("carrinho_alterar"))
        url = alterarCarrinho(req);
      else if (operacao.equals("pedido_gravar"))
        url = gravarPedido(req);
      else if (operacao.equals("pedidos_listar"))
        url = listarPedidos(req);
      else if (operacao.equals("pedido_excluir"))
        url = excluirPedido(req);
      else if (operacao.equals("busca"))
        url = buscarLivros(req);
      else throw new EditoraException("Operao invlida");
    }
    catch (Exception ex) {
      req.setAttribute("erro",ex);
      url = "/erro_tratado.jsp";
    }
    
    getServletContext().getRequestDispatcher(url).forward(req,res);
  }
  
  private void carregarMenu(HttpServletRequest req) 
    throws EditoraException {
    Vector assuntos = (Vector)AssuntoBO.listar();
    req.setAttribute("assunto_lista",assuntos);
  }
  
  private void carregarTitulo(HttpServletRequest req, String titulo)
    throws EditoraException {
    req.setAttribute("titulo",titulo);
  }
  
  private void carregarDestaque(HttpServletRequest req)
    throws EditoraException {
    Vector assuntos = (Vector)AssuntoBO.listar();
    Random r = new Random();
    int n = r.nextInt(assuntos.size());
    AssuntoBean ab = (AssuntoBean)assuntos.get(n);
    Vector livros = (Vector)LivroBO.selecionar(ab);
    n = r.nextInt(livros.size());
    LivroBean lb = (LivroBean)livros.get(n);
    livros = new Vector();
    livros.add(lb);
    req.setAttribute("livro_lista",livros);
  }
  
  private ClienteBean verificarPermissao(HttpServletRequest req)
    throws EditoraException {
    HttpSession sessao = req.getSession();
    ClienteBean b = (ClienteBean)sessao.getAttribute("cliente");
    String msg = "Efetue o logon antes de realizar esta operao";
    if (b == null) throw new EditoraException(msg);
    return b;
  }
  
  private String carregarInicio(HttpServletRequest req)
    throws EditoraException {
    carregarMenu(req);
    carregarTitulo(req, "Livro em destaque");
    carregarDestaque(req);
    return "/inicio.jsp";
  }
  
  private String carregarAssunto(HttpServletRequest req)
    throws EditoraException {
    AssuntoBean ab = new AssuntoBean();
    ab.setCodigo(req.getParameter("codigo"));
    ab = AssuntoBO.selecionar(ab.getCodigo());
    Vector livros = (Vector)LivroBO.selecionar(ab);
    
    carregarMenu(req);
    carregarTitulo(req, "Livros sobre " + ab.getDescricao());
    req.setAttribute("livro_lista",livros);
    return "/inicio.jsp";
 }
  
  private String incluirCliente(HttpServletRequest req)
    throws EditoraException {
    ClienteBean b = new ClienteBean(req.getParameter("email"),
      req.getParameter("senha"),req.getParameter("nome"));
        
    ClienteBO.incluir(b);
    req.getSession().setAttribute("cliente",b);
    return carregarInicio(req);
  }
  
  private String excluirCliente(HttpServletRequest req)
    throws EditoraException {
    ClienteBO.excluir(verificarPermissao(req));
    req.getSession().removeAttribute("cliente");
    return carregarInicio(req);
  }
  
  private String logon(HttpServletRequest req)
    throws EditoraException {
    ClienteBean b = new ClienteBean();
    b.setEmail(req.getParameter("email"));
    b.setSenha(req.getParameter("senha"));
    b = ClienteBO.validar(b);
    if (b == null) throw new EditoraException("Dados invlidos!");
    req.getSession().setAttribute("cliente",b);
    return carregarInicio(req);
  }
  
  private String logoff(HttpServletRequest req)
    throws EditoraException {
    req.getSession().invalidate();
    return carregarInicio(req);
  }
  
  private String exibirCarrinho(HttpServletRequest req)
    throws EditoraException {
    verificarPermissao(req);
    return "/carrinho.jsp";
  }
  
  private String incluirNoCarrinho(HttpServletRequest req)
    throws EditoraException {
    verificarPermissao(req);
    
    HttpSession sessao = req.getSession();
    Vector carrinho = (Vector)sessao.getAttribute("carrinho");
    if (carrinho == null) carrinho = new Vector();
    
    ItemBean b = new ItemBean();
    b.setIdLivro(req.getParameter("cd_livro"));
    LivroBean lb = LivroBO.selecionar(b.getIdLivro());
    b.setQuantidade(1);
    b.setUnitario(lb.getPreco());
    
    for(int i = 0; i < carrinho.size(); i++) {
      ItemBean ib = (ItemBean)carrinho.get(i);
      if (b.getIdLivro() == ib.getIdLivro())
        throw new EditoraException("O produto j est no carrinho");
    }
    
    carrinho.add(b);
    sessao.setAttribute("carrinho",carrinho);
    return "/carrinho.jsp";
  }
  
  private String excluirDoCarrinho(HttpServletRequest req)
    throws EditoraException {
    verificarPermissao(req);
    
    HttpSession sessao = req.getSession();
    Vector carrinho = (Vector)sessao.getAttribute("carrinho");
    if (carrinho == null) carrinho = new Vector();
    
    LivroBean livro = new LivroBean();
    livro.setCodigo(req.getParameter("cd_livro"));
    
    for(int i = 0; i < carrinho.size(); i++){
      ItemBean b = (ItemBean)carrinho.get(i);
      if (b.getIdLivro() == livro.getCodigo()) carrinho.remove(i--);
    }
    
    sessao.setAttribute("carrinho",carrinho);
    return "/carrinho.jsp";
  }
  
  private String alterarCarrinho(HttpServletRequest req)
    throws EditoraException {
    verificarPermissao(req);
    
    HttpSession sessao = req.getSession();
    Vector carrinho = (Vector)sessao.getAttribute("carrinho");
    if (carrinho == null) carrinho = new Vector();
    
    LivroBean livro = new LivroBean();
    livro.setCodigo(req.getParameter("cd_livro"));
    
    for(int i = 0; i < carrinho.size(); i++){
      ItemBean b = (ItemBean)carrinho.get(i);
      if (b.getIdLivro() == livro.getCodigo()) {
        b.setQuantidade(req.getParameter("qtde"));
        carrinho.set(i,b);
        break;
      }
    }
    
    sessao.setAttribute("carrinho",carrinho);
    req.setAttribute("url","carrinho.jsp");
    return "/sucesso.jsp";
  }
  
  private String gravarPedido(HttpServletRequest req)
    throws EditoraException {
    ClienteBean cb = verificarPermissao(req);
    
    HttpSession sessao = req.getSession();
    Vector carrinho = (Vector)sessao.getAttribute("carrinho");
    String msg = "O seu carrinho est vazio";
    if (carrinho == null) throw new EditoraException(msg);
    
    PedidoBean b = new PedidoBean();
    b.setData(new Date());
    b.setHorario(new Date());
    b.setCliente(cb);
    b.setItens(carrinho);
    PedidoBO.incluir(b);
    
    carrinho = new Vector();
    sessao.setAttribute("carrinho",carrinho);
    req.setAttribute("url","main");
    return "/sucesso.jsp";
  }
  
  private String listarPedidos(HttpServletRequest req)
    throws EditoraException {
    ClienteBean cb = verificarPermissao(req);
    Vector pedidos = (Vector)PedidoBO.selecionar(cb.getEmail());
    if (pedidos == null) pedidos = new Vector();
    req.setAttribute("pedido_lista",pedidos);
    return "/pedido_lista.jsp";
  }
  
  private String excluirPedido(HttpServletRequest req)
    throws EditoraException {
    ClienteBean cb = verificarPermissao(req);
    PedidoBean b = new PedidoBean();
    b.setNumero(req.getParameter("numero"));
    PedidoBO.excluir(b);
    Vector pedidos = (Vector)PedidoBO.selecionar(cb.getEmail());
    if (pedidos == null) pedidos = new Vector();
    req.setAttribute("pedido_lista",pedidos);
    return "/pedido_lista.jsp";
  }
  
  private String buscarLivros(HttpServletRequest req)
    throws EditoraException {
    String texto = req.getParameter("texto");
    if (texto == null || texto.trim().length() == 0)
      throw new EditoraException("Digite um texto para a busca");
    
    String filtro = req.getParameter("filtro");
    String msg = "Selecione uma opo de busca";
    if (filtro == null) throw new EditoraException(msg);
    
    Vector livros = new Vector();
    if (filtro.equals("Ttulo"))
      livros = (Vector)LivroBO.selecionar(texto);
    else if (filtro.equals("Autor")) {
      AutorBean b = new AutorBean();
      b.setNome(texto);
      livros = (Vector)LivroBO.selecionar(b);
    }
    
    carregarMenu(req);
    carregarTitulo(req,"Resultado da busca");
    req.setAttribute("livro_lista",livros);
    return "/inicio.jsp";
  }
}